﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CSI_L3_ADFGVXAlgorithm
{
    public partial class frmADFGVXEncrypt : Form
    {
        public frmADFGVXEncrypt()
        {
            InitializeComponent();
        }

        private void frmADFGVXEncrypt_Load(object sender, EventArgs e)
        {
            txtKey1.Text = "orange";
            txtKey2.Text = "water";
            txtText.Text = "cryptography";
            openFileDialog.FileName = "ADFGVXEncryptInput.txt";
            saveFileDialog.FileName = "ADFGVXEncryptOutput.txt";
        }

        private void btnEncrypt_Click(object sender, EventArgs e)
        {
            txtTextEncrypted.Text = ADFGVX.Encrypt(txtText.Text, txtKey1.Text, txtKey2.Text);


            //String sE = "";
            int[,] m = ADFGVX.Build7x7Matrix(ADFGVX.KeepOnlyLettersNumbers(txtKey1.Text));
            int l = m.GetLength(0);
            int c = m.GetLength(1);


            txtMatrix.Text = ADFGVX.matrixToString(m);
            for (int i = 0; i < ADFGVX.textEncrypted1Grouped.Length; i = i + 2)
                txtTextEncryptedGrouped.Text += "" + (char)ADFGVX.textEncrypted1Grouped[i] + "" + (char)ADFGVX.textEncrypted1Grouped[i + 1] + "  ";
            txtTranspositionMatrix.Text = ADFGVX.matrixToString(ADFGVX.transpositionMatrix);
            txtTranspositionMatrixOrdered.Text = ADFGVX.matrixToString(ADFGVX.transpositionMatrixOrdered);
        }

        //public string matrixToString(int[,] m)
        //{
        //    String s = "";
        //    for (int i = 0; i < m.GetLength(0); i++)
        //    {
        //        for (int j = 0; j < m.GetLength(1); j++)
        //            s += (char)m[i, j] + "  ";
        //        s += Environment.NewLine;
        //    }
        //    return s;
        //}

        private void btnLoadText_Click(object sender, EventArgs e)
        {
            txtText.Text = "";
            try
            {
                openFileDialog.InitialDirectory = Application.StartupPath;
                if (openFileDialog.ShowDialog() != null)
                {
                    String line;
                    StreamReader fileIn = new StreamReader(openFileDialog.FileName);

                    while ((line = fileIn.ReadLine()) != null)
                        txtText.Text += line;
                    fileIn.Close();
                }
            }
            catch (Exception ex) { MessageBox.Show("Load text: " + ex); }
        }

        private void btnEncryptFile_Click(object sender, EventArgs e)
        {
            openFileDialog.InitialDirectory = Application.StartupPath;
            saveFileDialog.InitialDirectory = Application.StartupPath;
            MessageBox.Show("Please select a text file in next appearing window:");
            if (openFileDialog.ShowDialog() != null)
            {
                MessageBox.Show("Please specify the name of the encrypted file in the next appearing window:");
                if (saveFileDialog.ShowDialog() != null)
                    try
                    {
                        String line;
                        StreamReader fileIn = new StreamReader(openFileDialog.FileName);
                        StreamWriter fileOut = new StreamWriter(saveFileDialog.FileName, false);
                        while ((line = fileIn.ReadLine()) != null)
                            fileOut.WriteLine("" + ADFGVX.Encrypt(line, txtKey1.Text, txtKey2.Text));
                        fileIn.Close(); fileOut.Close();
                    }

                    catch (Exception ex) { MessageBox.Show("Save text: " + ex); }

            }
        }


        private void btnOpenFolder_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start(Application.StartupPath);
        }
    }



    public static class ADFGVX
    {

        #region ADFGVX ENCRYPTION
        #endregion

        public static int[,] matrix7x7;
        public static int[,] transpositionMatrix;
        public static int[,] transpositionMatrixOrdered;
        public static String textEncrypted1Grouped;

        //#region MODULAR_ADFGVX_ALGORITHM
        public static string Encrypt(string text, string key1, string key2)
        {
            text = ADFGVX.KeepOnlyLettersNumbers(text);
            key1 = ADFGVX.KeepOnlyLettersNumbers(key1);
            key2 = ADFGVX.KeepOnlyLettersNumbers(key2);

            String sE = "";
            int[,] m = ADFGVX.Build7x7Matrix(key1);

            int n = m.GetLength(0);
            int i = 0, j = 0, k = 0;
            int i1 = 0, j1 = 0;
            bool charFound = false;

            // Encrypt text with first key1
            for (k = 0; k < text.Length; k++)
            {
                charFound = false;
                for (i = 1; i < n; i++)
                {
                    for (j = 1; j < n; j++)
                        if ((int)text[k] == m[i, j])
                        {
                            i1 = i;
                            j1 = j;
                            charFound = true;
                            break;
                        }
                    if (charFound == true)
                        break;
                }
                sE += (char)m[i1, 0]; sE += (char)m[0, j1];
            }
            textEncrypted1Grouped = sE;
            //MessageBox.Show("textEncryptedGrouped: " + sE);
            // MessageBox.Show("Encrypt 0.1 " + sE);



            // -------------------------------------------------------------

            // Encrypt text with second key2
            #region explanationEncryptKey2
            /*
             3.5 Sortarea celei de-a doua matrici şi construirea mesajului criptat
Vom adăuga o nouă coloană matricei precedente pe care o vom completa cu numere de la 1 ... N, unde N este lungimea celei de-a doua chei (lungimea cheii fară literele ce se repetă), după regula:
 se va cauta prima literă (în ordine alfabetică) a cuvantului iar pe coloana corespunzătoare se va scrie 1;
 se va căuta cea de-a doua literă să se va completa coloana cu 2;
 se va repeta procedeul până la epuizarea literelor celui de-al doilea cuvânt cheie.
Exemplu:
w D A A D D 5 
a D F A A V 1
t A V A F V 4
e D G V F A 2
r V D A V A 3

Mesajul criptat final va fi citit linie cu linie, în ordinea dată de a doua coloană:
a D F A A V 1
e D G V F A 2
r V D A V A 3
t A V A F V 4
w D A A D D 5

Mesajul criptat va fi citit apoi linie cu linie şi va fi transcris:
              
"DFAAVDGVFAVDAVAAVAFVDAADD"
 */
            #endregion

            int[,] m1 = ADFGVX.BuildTranspositionMatrix(sE, key2);
            transpositionMatrix = m1;
            // MessageBox.Show("Transposition matrix: " + matrixToString(transpositionMatrix));


            int l = m1.GetLength(0), c = m1.GetLength(1);
            int[,] m2 = new int[l, c];
            int[] order = new int[l];

            int aux;

            for (i = 0; i < l; i++)
                order[i] = m1[i, 0];

            // Key 2 ordered alphabetically
            for (i = 0; i < l - 1; i++)
                for (j = i + 1; j < l; j++)
                    if (order[i] > order[j])
                    { aux = order[i]; order[i] = order[j]; order[j] = aux; }

            // Crearea matricei finale: cu initialele liniilor ordonate alfabetic;

            // k = 0;
            for (i = 0; i < l; i++)
            {
                for (j = 0; j < l; j++)
                    if (m1[j, 0] == order[i])
                    {
                        for (j1 = 0; j1 < c; j1++)
                            m2[i, j1] = m1[j, j1];
                        //m2[i, c] = i;
                        break;
                    }
            }

            //MessageBox.Show("m2 " + matrixToString(m2));

            transpositionMatrixOrdered = m2;

            //Building ADFGVX Encrypted text;
            sE = "";
            for (i = 0; i < l; i++)
                for (j = 1; j < c; j++)
                    sE += (char)m2[i, j];

            //MessageBox.Show("transpositionMatrixOrdered: "+sE);
            return sE;
        }
        public static string matrixToString(int[,] m)
        {
            String s = "";
            for (int i = 0; i < m.GetLength(0); i++)
            {
                for (int j = 0; j < m.GetLength(1); j++)
                    s += (char)m[i, j] + "  ";
                s += Environment.NewLine;
            }
            return s;
        }
        public static int[,] BuildTranspositionMatrix(string encryptedKey1Text, string key2)
        {

            /*
              
             3.4 Construirea matricei de transpoziţie folosind cea de-a doua cheie:
            Vom constri o matrice ce are numărul de linii egale cu numărul de litere al celui de-al doilea cuvânt cheie. Vom scrie cuvântul cheie pe prima coloană a matricei. După care vom completa textul preliminar coloană cu coloană, de sus în jos şi de la stânga a dreapta. Dacă pe ultima coloană vor rămâne spaţii goale, le vom completa cu una din literele 'A', 'D', ... 'X' aleasă aleator.
            Pentru al doilea cuvânt cheie "water" şi textul preliminar "DD AD VA FV GD AA AV AD AF FV DV VA" vom obţine:
            
            w  D  A  A  D  D
            a  D  F  A  A  V
            t  A  V  A  F  V
            e  D  G  V  F  A
            r  V  D  A  V  A
                         
            */

            // Eliminating of characters repetition
            key2 = ADFGVX.EliminateCharRepetition(key2);

            //MessageBox.Show("key 2: "+key2);

            // Building transposition matrix

            int i = 0, j = 0, k = 0;
            int l = key2.Length;
            int c = (int)Math.Ceiling((double)encryptedKey1Text.Length / l) + 1;
            //MessageBox.Show("l={0} c={1} Etext={2}: " + l + " " + c + " " + encryptedKey1Text.Length + " ");
            int[,] m = new int[l, c];
            String a = "ADFGVX";
            Random obRnd = new Random();



            k = 0;
            for (i = 0; i < l; i++)
                m[i, 0] = (int)key2[k++];

            k = 0;
            for (j = 1; j < c; j++)
                for (i = 0; i < l; i++)
                    if (k >= encryptedKey1Text.Length)
                        m[i, j] = (int)a[obRnd.Next(0, a.Length)];
                    else
                        m[i, j] = (int)encryptedKey1Text[k++];
            return m;
        }

        public static int[,] Build7x7Matrix(string key1)
        {
            /*
             Criptarea ADFGVX implică folosirea a două chei diferite şi este recomandat ca fiecare dintre cele două chei sa aibă litere distincte. 
             Un exemplu de chei valide este: first şi second. Dacă însă unul dintre cele două cuvinte cheie are litere ce se repetă, se va elimina 
             cea de-a doua, a treaia, şamd apariţie din cuvănt: cheia „repetitie” va fi folosită în fapt ca „repti”.
 
            Criptarea ADFGVX implică parcurgerea următorilor paşi:
             alegearea cheilor şi prelucrarea acestora;
             construirea primei matrici (de substituţie) folosind prima cheie;
             criptarea preliminară a textului;
             construirea celei de-a doua matrici (de transpoziţie) folosind ce-a de-a doua cheie;
             sortarea celei de-a doua matrici;
             construirea mesajului criptat.
             * 
             3.2 Construirea matricei de substituţie folosind prima cheie
            Vom utiliza un tabel cu 7 linii si 7 coloane ce conţine atât pe prima linie cât şi pe
            prima coloana literele „_ADFGVX”. Vom completa cuvântul cheie pe prima linie a matricei
            de la stânga spre dreapta. Dacă lungimea cuvântului este mai mare decât şase, vom continua
            cu cea de-a doua linie şi aşa mai departe. Restul celulelor goale din matrice le vom completa
            în ordinea alfabetică, sărind peste cele care sunt deja prezente. Ultimele caractere introduse în
            matrice sunt cifrele de la '0' la '9'. 
            */

            /*
  
            Exemplu de matrice de substituţie (cheia = orange):
              A D F G V X
            A o r a n g e
            D b c d f h i
            F j k l m p q
            G s t u v w x
            V y z 0 1 2 3
            X 4 5 6 7 8 9
  
            */

            // Trim text: keep only [A-Z] and [0-9]
            // Eliminating of characters repetition
            String sE = ADFGVX.EliminateCharRepetition(ADFGVX.KeepOnlyLettersNumbers(key1));

            // Building 7x7 matrix
            String s = ADFGVX.KeyAddRemainingLettersNumbers(sE);

            // MessageBox.Show("7x7 Matrix: "+s);

            int i = 0, j = 0, k = 0;
            int n = 7;
            int[,] m = new int[n, n];

            String a = "ADFGVX";
            k = 0;
            m[0, 0] = (int)'A';
            for (i = 1; i < n; i++)
            {
                m[0, i] = (int)a[k];
                m[i, 0] = (int)a[k];
                k++;
            }

            //MessageBox.Show("7x7 Matrix 0: " + s);
            //String sTemp = "\n\n";
            k = 0;
            for (i = 1; i < n; i++)
            {
                for (j = 1; j < n; j++)
                    if ((i != 0) && (j != 0))
                    {
                        //sTemp += s[k];
                        m[i, j] = (int)s[k++];
                        //sTemp += //((char)m[i, j]);
                    } //sTemp += "\n";
            }
            //MessageBox.Show("Matrix 7x 7: "+sTemp); 
            return m;
        }

        public static string KeyAddRemainingLettersNumbers(string s)
        {
            int i = 0, j = 0;
            bool letterFound = false;

            // Add remaining [A-Z] letters 
            for (i = (int)'A'; i <= (int)'Z'; i++)
            {
                letterFound = false;
                for (j = 0; j < s.Length; j++)
                    if (s[j] == (char)i)
                    {
                        letterFound = true;
                        break;
                    }
                if (letterFound == false)
                    s += (char)i;
            }
            // Add remaining [0-9] numbers
            for (i = (int)'0'; i <= (int)'9'; i++)
            {
                letterFound = false;
                for (j = 0; j < s.Length; j++)
                    if (s[j] == (char)i)
                    {
                        letterFound = true;
                        break;
                    }
                if (letterFound == false)
                    s += (char)i;
            }
            return s;
        }

        public static string KeepOnlyLettersNumbers(string s)
        {
            String sE = "";
            s = s.ToUpper();

            // Trim: white spaces, number, small leters, ... : Trim all but letter in [A B ... Z]
            for (int i = 0; i < s.Length; i++)
                if ((((int)s[i] >= (int)'A') && ((int)s[i] <= (int)'Z')) || (((int)s[i] >= '0') && ((int)s[i] <= '9')))
                    sE += s[i];
            //MessageBox.Show("KeepOnlyLettersNumbers: sE:" + sE);
            return sE;
        }

        public static string EliminateCharRepetition(string sE)
        {
            string s = "";
            bool letterFound = false;
            for (int i = 0; i < sE.Length; i++)
            {
                letterFound = false;
                for (int j = 0; j < s.Length; j++)
                    if (s[j] == sE[i])
                    {
                        letterFound = true;
                        break;
                    }
                if (letterFound == false)
                    s += sE[i];
            }
            //MessageBox.Show("EliminateCharRepetition: s:"+s);
            return s;

        }















        #region ADFGVXDecryption
        #endregion



        public static string Decrypt(string text, string key1, string key2)
        {
            text = ADFGVX.KeepOnlyLettersNumbers(text);
            key1 = ADFGVX.KeepOnlyLettersNumbers(key1);
            key2 = ADFGVX.KeepOnlyLettersNumbers(key2);


            int i = 0, j = 0, k = 0, aux = 0, i1=0, j1 = 0;
            int l = key2.Length;
            int c = (int)Math.Ceiling((double)text.Length / l);
            int[] order = new int[l];
            int[,] m = new int[l, c];
            int[,] m1 = new int[l, c];
            int[,] m7x7 = ADFGVX.Build7x7Matrix(key1);
            matrix7x7 = m7x7;
            int n = m7x7.GetLength(0);
            String a = "ADFGVX";
            String s = "";

            k = 0;
            for (i = 0; i < l; i++)
                for (j = 0; j < c; j++)
                    m1[i, j] = text[k++];
            //MessageBox.Show(matrixToString(m1));
            transpositionMatrixOrdered = m1;
            for (i = 0; i < l; i++)
                order[i] = key2[i];

            // Key 2 ordered alphabetically
            for (i = 0; i < l - 1; i++)
                for (j = i + 1; j < l; j++)
                    if (order[i] > order[j])
                    { aux = order[i]; order[i] = order[j]; order[j] = aux; }

            // Crearea matricei cu initialele liniilor conform key2;
            k = 0;
            for (i = 0; i < l; i++)
            {
                for (j = 0; j < l; j++)
                    if (key2[i] == order[j])
                    {
                        for (j1 = 0; j1 < c; j1++)
                            m[i, j1] = m1[j, j1];
                        break;
                    }
            }

            //MessageBox.Show(matrixToString(m));
            transpositionMatrix = m;

            // Creating string from matrix m;
            for (j = 0; j < c; j++)
                for (i = 0; i < l; i++)
                    s += (char)m[i, j];
            //MessageBox.Show("S = " + s);

            // Decrypt text with first key1
            a = "";
            for (k = 0; k < s.Length - 1; k = k + 2)
            {
                for (i = 1; i < n; i++)
                {
                    if (m7x7[i,0] == (int)s[k])
                        i1 = i;
                    if (m7x7[0,i] == (int)s[k+1])
                        j1 = i;
                }
            a+=(char)m7x7[i1,j1];
            }
            //MessageBox.Show(matrixToString(m7x7));
            return a;
        }






    } // End class ADFGVX


}
