﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CSI_L5_ACAHomophonicAlgorithm
{
    public partial class frmACAHomophonicEncrypt : Form
    {
        public frmACAHomophonicEncrypt()
        {
            InitializeComponent();
        }

        private void frmACAHomophonicEncrypt_Load(object sender, EventArgs e)
        {
            openFileDialog.FileName = "input.txt";
            saveFileDialog.FileName = "HomophonicEncrypted.txt";

            txtKey.Text = "this";
            txtText.Text = "cryptography";
        }

        private void btnEncrypt_Click(object sender, EventArgs e)
        {
            String matrix = ACAHomophonic.matrixToString(ACAHomophonic.BuildEncryptionMatrix(txtKey.Text));
            String matrixTransposed = ACAHomophonic.matrixTransposedToString(ACAHomophonic.BuildEncryptionMatrix(txtKey.Text));
            String text = ACAHomophonic.Encrypt(txtText.Text, txtKey.Text);
            String nl = Environment.NewLine;

            txtLookupTable.Text = matrix + nl + nl + matrixTransposed;
            String s = "";

            for (int i = 0; i < text.Length; i = i + 2)
                s += "" + text[i] + text[i + 1] + " ";
            txtGroupedText.Text = s;
            txtTextEncrypted.Text = text;
        }

        private void btnEncryptFile_Click(object sender, EventArgs e)
        {
            String key = txtKey.Text;

            openFileDialog.InitialDirectory = Application.StartupPath;
            saveFileDialog.InitialDirectory = Application.StartupPath;
            MessageBox.Show("OPEN FILE: Please select a text file in next appearing window:");
            if (openFileDialog.ShowDialog() != null)
            {
                MessageBox.Show("SAVE FILE: Please specify the name of the encrypted file in the next appearing window:");
                if (saveFileDialog.ShowDialog() != null)
                    try
                    {
                        String line;
                        StreamReader fileIn = new StreamReader(openFileDialog.FileName);
                        StreamWriter fileOut = new StreamWriter(saveFileDialog.FileName, false);
                        while ((line = fileIn.ReadLine()) != null)
                            fileOut.WriteLine("" + ACAHomophonic.Encrypt(line, key));
                        fileIn.Close(); fileOut.Close();
                    }

                    catch (Exception ex) { MessageBox.Show("Save text: " + ex); }

            }
        }

        private void btnOpenFolder_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start(Application.StartupPath);
        }
    }
    
    #region ACA Homophonic Algorithm
    /*
 * 
    Cipher”. Această versiune criptează un alfabet de 25 de litere (în care I şi J sunt tratate în
acelaşi mod) la cifre de două litere.
 
Criptarea “ACA Homophonic” implică parcurgerea următorilor paşi:
 
 literele alfabetului sunt scrise pe o linie;
 se alege o cheie de patru litere;
 se scriu 4 linii după regula: dedesuptul primei litere din cheie se incepe numerotarea
cu 01 şi se continuă până la 25, dedesuptul celei de-a doua litere din cheie se (pe cea
de-a doua coloană de numere) se începe cu 26 şi se încheie cu 50, ş.a.m.d.
 pentru a cripta o literă se va alege aleator una dintre cele 4 numere aflate pe aceeaşi
coloană.
 
Exemplu de criptare:
 
Vom alege cheia: “THIS” şi vom construi matricea de criptare:
 
A  B  C  D  E  F  G  H I/J K  L  M  N  O  P  Q  R  S  T  U  V  W  X  Y  Z
08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 01 02 03 04 05 06 07
44 45 46 47 48 49 50 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43
68 69 70 71 72 73 74 75 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67
84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 00 76 77 78 79 80 81 82 83
  
*/
    #endregion

    public static class ACAHomophonic
    {
        public static String KeepOnlyLetters(string s)
        {
            String sE = "";
            s = s.ToUpper();

            // Trim: white spaces, number, small leters, ... : Trim all but letter in [A B ... Z]
            for (int i = 0; i < s.Length; i++)
                if (((int)s[i] >= (int)'A') && ((int)s[i] <= (int)'Z'))
                    sE += s[i];

            return sE;
        }
        public static int[,] BuildEncryptionMatrix(String key)
        {
            //Key : Must be 4 letters length; Only letters; UpperCase;

            int l = 5, col = 25;
            int[,] m = new int[l, col];
            int i, k, index;
            int num = 1;

            key = key.ToUpper();
            key = KeepOnlyLetters(key);
            if (key.Length == 4)
            {



                index = 0;
                for (i = (int)'A'; i <= (int)'Z'; i++)
                    if (i != (int)'J')
                        m[0, index++] = i;

                for (k = 0; k < key.Length; k++)
                {
                    for (i = 0; i < col; i++)
                        if (key[k] == (char)m[0, i])
                        {
                            index = i;
                            break;
                        }
                    for (i = index; i < col; i++)
                        m[k + 1, i] = num++;
                    for (i = 0; i < index; i++)
                        m[k + 1, i] = num++;
                }
                // Replace of 100 with 00.
                m[4, i - 1] = 0;



            }
            else
            { m = null; }
            return m;
        }
        public static String matrixToString(int[,] m)
        {
            String s = "";
            int i, j;
            String nl = Environment.NewLine;

            for (i = 0; i < m.GetLength(1); i++)
                s += "" + (char)m[0, i] + "   ";
            for (i = 1; i < m.GetLength(0); i++)
            {
                s += nl;
                for (j = 0; j < m.GetLength(1); j++)
                    if ((m[i, j] >= 0) && (m[i, j] <= 9))
                        s += "0" + m[i, j] + " ";
                    else
                        s += "" + m[i, j] + "  ";

            }

            return s;
        }
        public static String matrixTransposedToString(int[,] m)
        {
            String s = "";
            int i, j;
            String nl = Environment.NewLine;

            for (j = 0; j < m.GetLength(1); j++)
            {
                s += "" + (char)m[0, j] + " ";
                for (i = 1; i < m.GetLength(0); i++)
                    if ((m[i, j] >= 0) && (m[i, j] <= 9))
                        s += "0" + m[i, j] + " ";
                    else
                        s += "" + m[i, j] + "  ";
                s += nl;
            }

            return s;
        }
        public static String Encrypt(String text, String key)
        {

            String s = "";
            int j, k, index;
            int nr = 0;
            Random obRnd = new Random();
            text = KeepOnlyLetters(text);
            int[,] m = BuildEncryptionMatrix(key);


            for (k = 0; k < text.Length; k++)
            {

                for (j = 0; j < m.GetLength(1); j++)
                {

                    if (text[k] == (char)m[0, j])
                    {
                        index = obRnd.Next(1, m.GetLength(0));
                        nr = m[index, j];
                        break;
                    }

                }
                // if (nr != -1)
                if ((nr >= 0) && (nr <= 9))
                    s += "0" + nr;
                else
                    s += nr;
            }

            return s;
        }
        public static String Decrypt(String text, String key)
        {
            String s = "";
            int i, j, k;
            int nr = 0;
            bool found = true;
            int[,] m = BuildEncryptionMatrix(key);

            //MessageBox.Show("Encr mat: " + matrixTransposedToString(m));
            String sNr = "";
            int letter = 0;

            //MessageBox.Show("Text: " + text);
            try
            {
                for (k = 0; k < text.Length; k = k + 2)
                {
                    sNr = "" + text[k] + "" + text[k + 1];
                    nr = Convert.ToInt16(sNr);
                    found = false;
                    for (j = 0; j < m.GetLength(1); j++)
                    {
                        for (i = 1; i < m.GetLength(0); i++)
                            if (nr == m[i, j])
                            {
                                letter = m[0, j];
                                found = true;
                                break;
                            }
                        //if (found == true)
                        //    break;
                    }

                    s += (char)letter;
                }
            }
            catch (Exception e) { MessageBox.Show("Exception" + e); }

            //MessageBox.Show("DECR: " + s);
            return s;


        }

    } // end class ACAHomophonic
}
